<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:str="http://exslt.org/strings"
	extension-element-prefixes="str"
>
<xsl:strip-space elements="*" />
<xsl:output encoding="utf-8" method="text" />

<xsl:variable name="wiki_url">http://summersunset.delfi.lan/dokuwiki</xsl:variable>
<xsl:variable name="eventum_url">https://eventum.dev.delfi.ee</xsl:variable>

<xsl:template match="/xml">
	<!-- parse single node or all of them -->
	<xsl:choose>
		<xsl:when test="$id != ''">
			<xsl:apply-templates select="node[@id=$id]"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:apply-templates />
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="node">
	<xsl:apply-templates />
</xsl:template>

<xsl:template match="strong">**<xsl:apply-templates select="node()"/>**</xsl:template>
<xsl:template match="tt">''<xsl:apply-templates select="node()"/>''</xsl:template>
<xsl:template match="h1">&#10;====== <xsl:apply-templates select="node()"/> ======&#10;</xsl:template>
<xsl:template match="h2">&#10;===== <xsl:apply-templates select="node()"/> =====&#10;</xsl:template>
<xsl:template match="h3">&#10;==== <xsl:apply-templates select="node()"/> ====&#10;</xsl:template>
<xsl:template match="h4">&#10;=== <xsl:apply-templates select="node()"/> ===&#10;</xsl:template>
<xsl:template match="h5">&#10;== <xsl:apply-templates select="node()"/> ==&#10;</xsl:template>

<xsl:template match="p"><xsl:text>&#10;</xsl:text><xsl:apply-templates/><xsl:text>&#10;</xsl:text></xsl:template>

<xsl:template match="table">
	<xsl:apply-templates />
</xsl:template>

<xsl:template match="td">
<xsl:text>
|</xsl:text>
	<xsl:apply-templates />
<xsl:text>|</xsl:text>
</xsl:template>

<xsl:template match="link">
	<xsl:text>[[</xsl:text>


	<xsl:variable name="innerwiki_link">
		<xsl:call-template name="innerwiki_link">
			<xsl:with-param name="uri" select="@href"/>
		</xsl:call-template>
	</xsl:variable>

	<xsl:choose>
		<xsl:when test="$innerwiki_link != ''">
			<xsl:value-of select="$innerwiki_link" />
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="@href" />
		</xsl:otherwise>
	</xsl:choose>



	<xsl:value-of select="@idref" />

	<!-- an anchor -->
	<xsl:if test="@topicref != ''">
		<xsl:value-of select="//node/@id" />
		<xsl:text>#</xsl:text>
		<xsl:apply-templates />
	</xsl:if>

	<xsl:choose>
		<xsl:when test="normalize-space(.)">
			<xsl:text>|</xsl:text>
		</xsl:when>
		<xsl:otherwise>
			<xsl:if test="@idref != ''">
				<xsl:text>|</xsl:text>
				<xsl:call-template name="title">
					<xsl:with-param name="id" select="@idref" />
				</xsl:call-template>
			</xsl:if>

		</xsl:otherwise>
	</xsl:choose>

	<xsl:apply-templates />

	<xsl:text>]]</xsl:text>
</xsl:template>

<xsl:template match="list|toc">
	<xsl:apply-templates />
</xsl:template>

<xsl:template match="ul|ol">
	<xsl:apply-templates />
</xsl:template>

<xsl:template match="li">
	<xsl:text>&#10;</xsl:text>

	<xsl:value-of select="str:padding(count(ancestor::ul) * 2 + count(ancestor::ol) * 2, '  ')" />
	<xsl:if test="name(..) = 'ul'">
		<xsl:text>* </xsl:text>
	</xsl:if>
	<xsl:if test="name(..) = 'ol'">
		<xsl:text>- </xsl:text>
	</xsl:if>

	<xsl:apply-templates />
</xsl:template>

<xsl:template match="code">
	<xsl:text>&lt;code&gt;</xsl:text>
		<xsl:value-of select="node()" />
	<xsl:text>&lt;/code&gt;&#10;</xsl:text>
</xsl:template>

<xsl:template match="pre">
	<xsl:text>&#10;''</xsl:text>
		<xsl:value-of select="node()" />
	<xsl:text>''&#10;</xsl:text>
</xsl:template>

<xsl:template match="em">
	<xsl:text>//</xsl:text>
		<xsl:value-of select="node()" />
	<xsl:text>//</xsl:text>
</xsl:template>

<xsl:template match="hr">
	<xsl:text>&#10;&#10;----</xsl:text>
</xsl:template>

<!--
Process cowiki variables:
<var name="TITLE"/>
<var name="PARENT_TITLE"/>
-->
<xsl:template match="var">
	<xsl:choose>
		<xsl:when test="@name = 'TITLE'">
			<xsl:call-template name="title">
				<xsl:with-param name="id" select="$id" />
			</xsl:call-template>
		</xsl:when>

		<xsl:when test="@name = 'PARENT_TITLE'">
			<xsl:variable name="parent_id">
					<xsl:value-of select="/xml/node[@id=$id]/@parent_id" />
			</xsl:variable>
			<xsl:call-template name="title">
				<xsl:with-param name="id" select="$parent_id" />
			</xsl:call-template>
		</xsl:when>

		<xsl:otherwise>
			Unknown variable <xsl:value-of select="@name" />
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!--
Process plugins:
<plugin name="DocRecent" limit="200" cutoff="80" title="Recent Changes"/>
-->
<xsl:template match="plugin">
	<xsl:choose>
		<xsl:when test="@name = 'DocRecent'">
			<xsl:value-of select="/xml/node[@id=$id]/@name" />
			<!--- make docrecent as rss feed -->
			<xsl:text>{{rss></xsl:text>
			<xsl:value-of select="$wiki_url" />
			<xsl:text>/feed.php </xsl:text>
			<xsl:value-of select="@cutoff" />
			<xsl:text> author date 1h}}</xsl:text>
		</xsl:when>

		<xsl:otherwise>
			<xsl:text>==== Unknown plugin </xsl:text>
			<xsl:value-of select="@name" />
			<xsl:text> ====&#10;</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="uri">
	<xsl:text>[[</xsl:text>
	<xsl:value-of select="@strref" />
	<xsl:text>]]</xsl:text>
</xsl:template>

<!--- char *title(int node_id) -->
<xsl:template name="title">
	<xsl:param name="id" />
	<xsl:value-of select="/xml/titles/node[@id=$id]/@title"/>
</xsl:template>

<!-- returns hostname from $url -->
<xsl:template name="hostname">
	<xsl:param name="uri"/>
	<xsl:for-each select="str:tokenize($uri, '/')">
		<xsl:if test="position() = 2">
			<xsl:value-of select="." />
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<!-- mungles link if it is one innerwiki link, eventum, cvs, ... -->
<xsl:template name="innerwiki_link">
	<xsl:param name="uri"/>

	<xsl:variable name="hostname">
		<xsl:call-template name="hostname">
			<xsl:with-param name="uri" select="@href"/>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name="eventum_host">
		<xsl:call-template name="hostname">
			<xsl:with-param name="uri" select="$eventum_url"/>
		</xsl:call-template>
	</xsl:variable>

	<!--
		Convert to eventum links:
		<link href="https://eventum.dev.delfi.ee/view.php?id=458">458</link>
		[[issue>354|issue #354]]
	-->
	<xsl:if test="$hostname = $eventum_host">
		<xsl:text>issue&gt;</xsl:text>
		<xsl:for-each select="str:tokenize($uri, '=')">
			<xsl:if test="position() = 2">
				<xsl:value-of select="." />
			</xsl:if>
		</xsl:for-each>
	</xsl:if>
</xsl:template>

</xsl:stylesheet>
